﻿
Imports Microsoft.Xna.Framework.Graphics
Imports Microsoft.Xna.Framework
Imports FarseerPhysics.Common
Imports FarseerPhysics.Common.Decomposition
Imports FarseerPhysics.Common.PolygonManipulation
Imports FarseerPhysics.Dynamics
Imports FarseerPhysics.Factories


Public Class DrawablePhysicsObject
    ' Because Farseer uses 1 unit = 1 meter we need to convert
    ' between pixel coordinates and physics coordinates.
    ' I've chosen to use the rule that 100 pixels is one meter.
    ' We have to take care to convert between these two 
    ' coordinate-sets wherever we mix them!

    Public Const unitToPixel As Single = 100.0F
    Public Const pixelToUnit As Single = 1 / unitToPixel

    Public body As Body
    Private scale As New Vector2
    Public Property Position() As Vector2
        Get
            Return body.Position * unitToPixel
        End Get
        Set(value As Vector2)
            body.Position = value * pixelToUnit
        End Set
    End Property

    Public texture As Texture2D

    Private m_size As Vector2
    Public Property Size() As Vector2
        Get
            Return m_size * unitToPixel
        End Get
        Set(value As Vector2)
            m_size = value * pixelToUnit
        End Set
    End Property

    ''' <param name="world">The farseer simulation this object should be part of</param>
    ''' <param name="texture">The image that will be drawn at the place of the body</param>
    ''' <param name="size">The size in pixels</param>
    ''' <param name="mass">The mass in kilograms</param>
    Public Sub New(world As World, texture As Texture2D, size As Vector2, mass As Single, type As Integer)
        Me.Size = size
        Me.texture = texture
        scale = New Vector2(size.X / CSng(texture.Width), size.Y / CSng(texture.Height))

        If type = 0 Then
            body = BodyFactory.CreateRectangle(world, size.X * pixelToUnit, size.Y * pixelToUnit, 1)
            body.Mass = mass
        End If

        If type = 1 Then
            body = BodyFactory.CreateCircle(world, 0.17, 1)
            body.Mass = mass
        End If

        If type = 2 Then
            body = createfromtexture(world, texture)
            body.Mass = mass
        End If
    End Sub


    Public Sub Draw(spriteBatch As SpriteBatch)
        'the texture has no link whatsoever with our world - it is there only to draw
        spriteBatch.Draw(texture, Position, Nothing, Color.White, body.Rotation, New Vector2(texture.Width / 2.0F, texture.Height / 2.0F), scale, SpriteEffects.None, 0)

    End Sub

    Public Sub draw(spriteBatch As SpriteBatch, frame As Integer)
        spriteBatch.Draw(texture, Position, New Rectangle(frame * 28, 0, 28, 32), Color.White, body.Rotation, New Vector2(28 / 2, 32 / 2), 1, SpriteEffects.None, 0)
    End Sub

    Private Function createfromtexture(world As World, texture As Texture2D) As Body

        'load texture that will represent the physics body
        Dim _polygonTexture As Texture2D = texture

        'Create an array to hold the data from the texture
        Dim data As UInteger() = New UInteger(_polygonTexture.Width * _polygonTexture.Height - 1) {}

        'Transfer the texture data to the array
        _polygonTexture.GetData(data)

        'Find the vertices that makes up the outline of the shape in the texture
        Dim textureVertices As Vertices = PolygonTools.CreatePolygon(data, _polygonTexture.Width, False)

        'The tool return vertices as they were found in the texture.
        'We need to find the real center (centroid) of the vertices for 2 reasons:

        '1. To translate the vertices so the polygon is centered around the centroid.
        Dim centroid As Vector2 = -textureVertices.GetCentroid()
        textureVertices.TranslateVB_byval(centroid)

        '2. To draw the texture the correct place.
        Dim _origin As Vector2 = -centroid

        'We simplify the vertices found in the texture.
        textureVertices = SimplifyTools.ReduceByDistance(textureVertices, 4.0F)

        'Since it is a concave polygon, we need to partition it into several smaller convex polygons
        Dim list As List(Of Vertices) = Triangulate.ConvexPartition(textureVertices, TriangulationAlgorithm.Bayazit)

        Dim _scale As Single = 1.0F

        'scale the vertices from graphics space to sim space
        Dim vertScale As Vector2 = New Vector2(ConvertUnits.ToSimUnits(1)) * scale
        For Each vertices As Vertices In list
            vertices.ScaleVB_byval(vertScale)
        Next

        'Create a single body with multiple fixtures
        Dim _compound As Body = BodyFactory.CreateCompoundPolygon(world, list, 1.0F)
        Return _compound
    End Function


    
End Class
